function startFullScreen(element) {
  if (element.requestFullscreen) {
    element.requestFullscreen();
  } else if (element.mozRequestFullScreen) {
    element.mozRequestFullScreen();
  } else if (element.webkitRequestFullscreen) {
    element.webkitRequestFullscreen();
  } else if (element.msRequestFullscreen) {
    element.msRequestFullscreen();
  }
}

function cancelFullScreen(element) {
  if (document.exitFullscreen) {
    document.exitFullscreen();
  } else if (document.mozCancelFullScreen) {
    document.mozCancelFullScreen();
  } else if (document.webkitCancelFullScreen) {
    document.webkitCancelFullScreen();
  } else if (document.msExitFullscreen) {
    document.msExitFullscreen();
  }
}

function toggleFullScreen(element) {
  var isInFullScreen = (document.fullscreenElement && document.fullscreenElement !== null) ||
        (document.webkitFullscreenElement && document.webkitFullscreenElement !== null) ||
        (document.mozFullScreenElement && document.mozFullScreenElement !== null) ||
        (document.msFullscreenElement && document.msFullscreenElement !== null);
  if (!isInFullScreen) {
    startFullScreen(element);
  } else {
    cancelFullScreen();
  }
}

function removeElement(element) {
  element && element.parentNode && element.parentNode.removeChild(element);
}

function hideElement(element) {
  if (element)
    element.style.display = 'none';
}

function showElement(element) {
  if (element)
    element.style.display = 'block';
}

function showHideElement(element) {
  if (element) {
    if (element.style.display == 'block')
      element.style.display = 'none';
    else
      element.style.display = 'block';
  }
}

function hideElement2(element) {
  if (element)
    element.style.visibility = "hidden";
}

function showElement2(element) {
  if (element)
    element.style.visibility = "visible";
}


function getStatusGlobal() {
  var xhttp = new XMLHttpRequest();
  xhttp.onreadystatechange = function() {
    if (xhttp.readyState === 4) {
      var backgroundModeStart = document.getElementById("backgroundModeStart");
      var backgroundModeStop = document.getElementById("backgroundModeStop");
      var backgroundModeStatus = document.getElementById("backgroundModeStatus");
      var cpuUsagePercents = document.getElementById("cpuUsagePercents");
      var cpuFrequencyMhz = document.getElementById("cpuFrequencyMhz");
      var liveConnections = document.getElementById("liveConnections");
      var battery = document.getElementById("battery");
      var memoryUsed = document.getElementById("memoryUsed");
      var memoryFree = document.getElementById("memoryFree");
      var threadsUsed = document.getElementById("threadsUsed");
      var threadsRunnableUsed = document.getElementById("threadsRunnableUsed");
//      var processes = document.getElementById("processes");
//      var uptime = document.getElementById("uptime");
      var networkIn = document.getElementById("networkIn");
      var networkOut = document.getElementById("networkOut");
      var recorded = document.getElementById("recorded");
      var available = document.getElementById("available");
      var streamProfile = document.getElementById("streamProfile");
      var powerSafeMode = document.getElementById("powerSafeMode");
      var notifications = document.getElementById("notifications");
      var notificationsWarning = document.getElementById("notificationsWarning");
      var motion = document.getElementById("motion");
      var temperature = document.getElementById("temperature");
      if (xhttp.status === 200) {
        var json = JSON.parse(xhttp.responseText).data;
        var backgroundMode = json.backgroundMode;
        backgroundModeStart.disabled = backgroundMode;
        backgroundModeStop.disabled = !backgroundMode;
        backgroundModeStatus.textContent = (backgroundMode ? "Started" : "Stopped");
        cpuUsagePercents.textContent = json.cpuUsagePercents;
        cpuFrequencyMhz.textContent = json.cpuFrequencyMhz;
        liveConnections.textContent = json.liveConnections;
        battery.textContent = json.batteryLevel;
//        uptime.textContent = json.uptime;
        networkIn.textContent = json.networkInBps;
        networkOut.textContent = json.networkOutBps;
        recorded.textContent = Math.floor(json.spaceUsed / 1024 / 1024);
        available.textContent = Math.floor(json.spaceAvailable / 1024 / 1024);
        memoryFree.textContent = Math.floor(json.memoryAvailable / 1024 / 1024);
        memoryUsed.textContent = Math.floor(json.memoryUsed / 1024 / 1024);
        if (temperature != null)
          temperature.textContent = json.temperature;
        if (threadsUsed != null)
          threadsUsed.textContent = json.threadsUsed;
        if (threadsRunnableUsed != null)
          threadsRunnableUsed.textContent = json.threadsRunnableUsed;
//        if (processes != null)
//          processes.textContent = json.processes;
        cpuUsageSeries.append(new Date().getTime(), json.cpuUsagePercents);
        cpuFreqSeries.append(new Date().getTime(), json.cpuFrequencyMhz);
        streamProfile.selectedIndex = json.streamProfile;
        powerSafeMode.selectedIndex = json.powerSafeMode ? 1 : 0;
        notifications.selectedIndex = json.notifications ? 1 : 0;
        notificationsWarning.style.display = (json.notifications == 1 ? 'none' : 'block');
        motion.textContent = json.motion;
        setTimeout(getStatusGlobal, 2500);
      } else {
        console.warn('tinyCam web server is not responding. Retry in 10 sec.');
//        temperature.style.visibility = "hidden";
        backgroundModeStart.disabled = true;
        backgroundModeStop.disabled = true;
        backgroundModeStatus.textContent = "-";
        cpuFrequencyMhz.textContent = "-";
        cpuUsagePercents.textContent = "-";
        liveConnections.textContent = "-";
        battery.textContent = "-";
//        uptime.textContent = "-";
        networkIn.textContent = "-";
        networkOut.textContent = "-";
        recorded.textContent = "-";
        available.textContent = "-";
        memoryFree.textContent = "-";
        memoryUsed.textContent = "-";
        if (temperature != null)
          temperature.textContent = "-";
        if (threadsUsed != null)
          threadsUsed.textContent = "-";
        if (threadsRunnableUsed != null)
          threadsRunnableUsed.textContent = "-";
//        if (processes != null)
//          processes.textContent = "-";
        cpuUsageSeries.append(new Date().getTime(), -1);
        cpuFreqSeries.append(new Date().getTime(), -1);
        streamProfile.selectedIndex = -1;
        powerSafeMode.selectedIndex = -1;
        notifications.selectedIndex = -1;
        notificationsWarning.style.display = 'none';
        motion.textContent = "-";
        setTimeout(getStatusGlobal, 10000);
      }
    }
  };
  xhttp.open("GET", "/api/v1/get_status", true);
  xhttp.send();
}

function getStatusCamera(camId) {
  var xhttp = new XMLHttpRequest();
  xhttp.onreadystatechange = function() {
    if (xhttp.readyState === 4) {
      var motion = document.getElementById("motion");
      if (xhttp.status === 200) {
        var json = JSON.parse(xhttp.responseText).data;
        setTimeout(getStatusCamera, 2500, camId);
        if (json.motion) {
          showElement2(document.getElementById('icon_md_on'));
          hideElement2(document.getElementById('icon_md_off'));
        } else {
          showElement2(document.getElementById('icon_md_off'));
          hideElement2(document.getElementById('icon_md_on'));
        }
      } else {
        hideElement2(document.getElementById('icon_md_on'));
        hideElement2(document.getElementById('icon_md_off'));
        console.warn('tinyCam web server is not responding. Retry in 10 sec.');
        setTimeout(getStatusCamera, 10000, camId);
        setTimeout(loadImage, 10000, 0, true);
      }
    }
  };
  xhttp.open("GET", "/api/v1/get_status?cameraId=" + camId, true);
  xhttp.send();
}

function fitToContainerWidth(canvas) {
  canvas.style.width='100%';
  canvas.width = canvas.offsetWidth;
}

function sendHttpGetAsync(request) {
  var xhttp = new XMLHttpRequest();
  xhttp.open("GET", request, true);
  xhttp.send();
}

function addRemoveElementListener(element, callbackRemoved) {
  var in_dom = document.body.contains(element);
  var observer = new MutationObserver(function(mutations) {
    if (element.closest('body')) {
      in_dom = true;
    } else if (in_dom) {
      in_dom = false;
      callbackRemoved(element);
    }
  });
  observer.observe(document.body, {childList: true});
}

function loadImage(cellId) {
  loadImage(cellId, false);
}

function loadImage(cellId, forceReload) {
//console.log('loadImage(cellId=' + cellId + ')');
  var image = document.getElementById("image" + cellId);
  var cell  = document.getElementById("cell"  + cellId);
  var url = (forceReload ? image.src + '&' + randomInteger(0, 10000) : image.src);
  cell.style.backgroundImage = "url('" + url + "')";
//setTimeout(loadImage, 5000, cellId);
}

function loadImageError(cellId) {
  console.log('loadImageError(cellId=' + cellId + ')');
  cell.style.backgroundImage = "url('/static/loading.png')";
  loadImage(cellId, true);
}

function randomInteger(min, max) {
  var rand = min - 0.5 + Math.random() * (max - min + 1)
  return Math.round(rand);
}

function logout(secUrl) {
  window.location = window.location.protocol + '//logout:' + (new Date()).getTime().toString() + '@' + window.location.host + '/';
}
